@@ -2,8 +2,9 @@
  *  Copyright (C) 2001-2003 Matthieu Sozeau <mattam@altern.org>
  *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
  *                           Thomas Nilsson and 4Front Technologies
- *  Copyright (C) 1999-2005  Haavard Kvaalen
+ *  Copyright (C) 1999-2006  Haavard Kvaalen
  *  Copyright (C) 2005       Takashi Iwai
+ *  Copyright (C) 2007       William Pitcock
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,6 +32,7 @@
 
 #include "alsa.h"
 #include <ctype.h>
+#include <math.h>
 #include <audacious/xconvert.h>
 
 static snd_pcm_t *alsa_pcm;
@@ -64,7 +66,7 @@ static int rd_index, wr_index;	 /* current read/write position in int-buffer */
 static gboolean pause_request;	 /* pause status currently requested */
 static int flush_request;	 /* flush status (time) currently requested */
 static int prebuffer_size;
-GMutex *alsa_mutex;
+GStaticMutex alsa_mutex = G_STATIC_MUTEX_INIT;
 
 static guint mixer_timeout;
 
@@ -131,11 +133,15 @@ int alsa_playing(void)
 	return snd_pcm_state(alsa_pcm) == SND_PCM_STATE_RUNNING;
 }
 
-static int xrun_recover(void)
+static int
+alsa_recovery(int err)
 {
+	int err2;
+
+	/* if debug mode is enabled, dump ALSA state to console */
 	if (alsa_cfg.debug)
 	{
-		snd_pcm_status_t *alsa_status;
+		snd_pcm_status_t *alsa_status = NULL;
 		snd_pcm_status_alloca(&alsa_status);
 		if (snd_pcm_status(alsa_pcm, alsa_status) < 0)
 			g_warning("xrun_recover(): snd_pcm_status() failed");
@@ -145,37 +151,39 @@ static int xrun_recover(void)
 			snd_pcm_status_dump(alsa_status, logs);
 		}
 	}
-	return snd_pcm_prepare(alsa_pcm);
-}
 
-static int suspend_recover(void)
-{
-	int err;
-
-	while ((err = snd_pcm_resume(alsa_pcm)) == -EAGAIN)
-		/* wait until suspend flag is released */
-		xmms_usleep(1000000);
-	if (err < 0)
-	{
-		g_warning("alsa_handle_error(): "
-			  "snd_pcm_resume() failed.");
-		return snd_pcm_prepare(alsa_pcm);
-	}
-	return err;
-}
-
-/* handle generic errors */
-static int alsa_handle_error(int err)
-{
+	/*
+	 * specifically handle -EPIPE and -ESTRPIPE to recover 
+	 * PCM fragment periods without losing data.
+	 */
 	switch (err)
 	{
-		case -EPIPE:
-			return xrun_recover();
-		case -ESTRPIPE:
-			return suspend_recover();
+	case -ESTRPIPE:
+	case ESTRPIPE:   /* "suspend": wait until ALSA is "running" again. */
+		while ((err2 = snd_pcm_resume(alsa_pcm)) == -EAGAIN)
+			g_usleep(100000);
+
+		if (err2 < 0)
+			return snd_pcm_prepare(alsa_pcm);
+
+		break;
+
+	case -EPIPE:
+	case EPIPE:      /* under-run and the I/O pipe closed on us */
+		return snd_pcm_prepare(alsa_pcm);
+		break;
+
+	case EINTR:
+	case -EINTR:
+		break;
+
+	default:
+		g_warning("Unhandled ALSA exception code %d (%s), trying hard restart.", err, snd_strerror(err));
+		return snd_pcm_prepare(alsa_pcm);
+		break;
 	}
 
-	return err;
+	return 0;
 }
 
 /* update and get the available space on h/w buffer (in frames) */
@@ -188,11 +196,11 @@ static snd_pcm_sframes_t alsa_get_avail(void)
 
 	while ((ret = snd_pcm_avail_update(alsa_pcm)) < 0)
 	{
-		ret = alsa_handle_error(ret);
+		ret = alsa_recovery(ret);
 		if (ret < 0)
 		{
 			g_warning("alsa_get_avail(): snd_pcm_avail_update() failed: %s",
-				  snd_strerror(-ret));
+				  snd_strerror(ret));
 			return 0;
 		}
 	}
@@ -247,7 +255,7 @@ static void alsa_close_pcm(void)
 		snd_pcm_drop(alsa_pcm);
 		if ((err = snd_pcm_close(alsa_pcm)) < 0)
 			g_warning("alsa_pcm_close() failed: %s",
-				  snd_strerror(-err));
+				  snd_strerror(err));
 		alsa_pcm = NULL;
 	}
 }
@@ -263,11 +271,11 @@ void alsa_close(void)
 
 	g_thread_join(audio_thread);
 
-	g_mutex_lock(alsa_mutex); /* alsa_loop locks alsa_mutex! */
+	g_static_mutex_lock(&alsa_mutex); /* alsa_loop locks alsa_mutex! */
 
 	alsa_cleanup_mixer();
 
-	xmms_convert_buffers_destroy(convertb);
+	aud_convert_buffers_destroy(convertb);
 	convertb = NULL;
 	g_free(inputf);
 	inputf = NULL;
@@ -282,7 +290,7 @@ void alsa_close(void)
 		snd_output_close(logs);
 	debug("Device closed");
 
-	g_mutex_unlock(alsa_mutex);
+	g_static_mutex_unlock(&alsa_mutex);
 }
 
 /* reopen ALSA PCM */
@@ -315,7 +323,7 @@ void alsa_flush(int time)
 {
 	flush_request = time;
 	while (flush_request != -1)
-		xmms_usleep(10000);
+		g_usleep(10000);
 }
 
 static void parse_mixer_name(char *str, char **name, int *index)
@@ -345,36 +353,37 @@ int alsa_get_mixer(snd_mixer_t **mixer, int card)
 
 	debug("alsa_get_mixer");
 
-	dev = g_strdup_printf("hw:%i", card);
-
 	if ((err = snd_mixer_open(mixer, 0)) < 0)
 	{
 		g_warning("alsa_get_mixer(): Failed to open empty mixer: %s",
-			  snd_strerror(-err));
+			  snd_strerror(err));
 		mixer = NULL;
 		return -1;
 	}
+
+	dev = g_strdup_printf("hw:%i", card);
 	if ((err = snd_mixer_attach(*mixer, dev)) < 0)
 	{
 		g_warning("alsa_get_mixer(): Attaching to mixer %s failed: %s",
-			  dev, snd_strerror(-err));
+			  dev, snd_strerror(err));
+		g_free(dev);
 		return -1;
 	}
+	g_free(dev);
+
 	if ((err = snd_mixer_selem_register(*mixer, NULL, NULL)) < 0)
 	{
 		g_warning("alsa_get_mixer(): Failed to register mixer: %s",
-			  snd_strerror(-err));
+			  snd_strerror(err));
 		return -1;
 	}
 	if ((err = snd_mixer_load(*mixer)) < 0)
 	{
 		g_warning("alsa_get_mixer(): Failed to load mixer: %s",
-			  snd_strerror(-err));
+			  snd_strerror(err));
 		return -1;
 	}
 
-	g_free(dev);
-
 	return (*mixer != NULL);
 }
 
@@ -440,8 +449,7 @@ static int alsa_setup_mixer(void)
 		return -1;
 	}
 
-	if (!alsa_cfg.soft_volume)
-		alsa_set_volume(a * 100 / alsa_max_vol, b * 100 / alsa_max_vol);
+	alsa_set_volume(a * 100 / alsa_max_vol, b * 100 / alsa_max_vol);
 
 	debug("alsa_setup_mixer: end");
 
@@ -459,7 +467,6 @@ static int alsa_mixer_timeout(void *data)
 	mixer_timeout = 0;
 	mixer_start = TRUE;
 
-	g_message("alsa mixer timed out");
 	return FALSE;
 }
 
@@ -483,28 +490,20 @@ void alsa_get_volume(int *l, int *r)
 		mixer_start = FALSE;
 	}
 
-	if (alsa_cfg.soft_volume)
-	{
-		*l = alsa_cfg.vol.left;
-		*r = alsa_cfg.vol.right;
-	}
-
 	if (!pcm_element)
 		return;
 
 	snd_mixer_handle_events(mixer);
 
-	if (!alsa_cfg.soft_volume)
-	{
-		snd_mixer_selem_get_playback_volume(pcm_element,
-						    SND_MIXER_SCHN_FRONT_LEFT,
-						    &ll);
-		snd_mixer_selem_get_playback_volume(pcm_element,
-						    SND_MIXER_SCHN_FRONT_RIGHT,
-						    &lr);
-		*l = ll;
-		*r = lr;
-	}
+	snd_mixer_selem_get_playback_volume(pcm_element,
+					    SND_MIXER_SCHN_FRONT_LEFT,
+					    &ll);
+	snd_mixer_selem_get_playback_volume(pcm_element,
+					    SND_MIXER_SCHN_FRONT_RIGHT,
+					    &lr);
+	*l = ll;
+	*r = lr;
+
 	if (mixer_timeout)
 		gtk_timeout_remove(mixer_timeout);
 	mixer_timeout = gtk_timeout_add(5000, alsa_mixer_timeout, NULL);
@@ -513,13 +512,6 @@ void alsa_get_volume(int *l, int *r)
 
 void alsa_set_volume(int l, int r)
 {
-	if (alsa_cfg.soft_volume)
-	{
-		alsa_cfg.vol.left = l;
-		alsa_cfg.vol.right = r;
-		return;
-	}
-
 	if (!pcm_element)
 		return;
 
@@ -588,108 +580,6 @@ int alsa_get_written_time(void)
 	return (alsa_total_written * 1000) / inputf->bps;
 }
 
-#define STEREO_ADJUST(type, type2, endian)					\
-do {										\
-	type *ptr = data;							\
-	for (i = 0; i < length; i += 4)						\
-	{									\
-		*ptr = type2##_TO_##endian(type2##_FROM_## endian(*ptr) *	\
-					   alsa_cfg.vol.left / 100);		\
-		ptr++;								\
-		*ptr = type2##_TO_##endian(type2##_FROM_##endian(*ptr) *	\
-					   alsa_cfg.vol.right / 100);		\
-		ptr++;								\
-	}									\
-} while (0)
-
-#define MONO_ADJUST(type, type2, endian)					\
-do {										\
-	type *ptr = data;							\
-	for (i = 0; i < length; i += 2)						\
-	{									\
-		*ptr = type2##_TO_##endian(type2##_FROM_## endian(*ptr) *	\
-					   vol / 100);				\
-		ptr++;								\
-	}									\
-} while (0)
-
-#define VOLUME_ADJUST(type, type2, endian)		\
-do {							\
-	if (channels == 2)				\
-		STEREO_ADJUST(type, type2, endian);	\
-	else						\
-		MONO_ADJUST(type, type2, endian);	\
-} while (0)
-
-#define STEREO_ADJUST8(type)				\
-do {							\
-	type *ptr = data;				\
-	for (i = 0; i < length; i += 2)			\
-	{						\
-		*ptr = *ptr * alsa_cfg.vol.left / 100;	\
-		ptr++;					\
-		*ptr = *ptr * alsa_cfg.vol.right / 100;	\
-		ptr++;					\
-	}						\
-} while (0)
-
-#define MONO_ADJUST8(type)			\
-do {						\
-	type *ptr = data;			\
-	for (i = 0; i < length; i++)		\
-	{					\
-		*ptr = *ptr * vol / 100;	\
-		ptr++;				\
-	}					\
-} while (0)
-
-#define VOLUME_ADJUST8(type)			\
-do {						\
-	if (channels == 2)			\
-		STEREO_ADJUST8(type);		\
-	else					\
-		MONO_ADJUST8(type);		\
-} while (0)
-
-
-static void volume_adjust(void* data, int length, AFormat fmt, int channels)
-{
-	int i, vol;
-
-	if ((alsa_cfg.vol.left == 100 && alsa_cfg.vol.right == 100) ||
-	    (channels == 1 &&
-	     (alsa_cfg.vol.left == 100 || alsa_cfg.vol.right == 100)))
-		return;
-
-	vol = MAX(alsa_cfg.vol.left, alsa_cfg.vol.right);
-
-	switch (fmt)
-	{
-		case FMT_S16_LE:
-			VOLUME_ADJUST(gint16, GINT16, LE);
-			break;
-		case FMT_U16_LE:
-			VOLUME_ADJUST(guint16, GUINT16, LE);
-			break;
-		case FMT_S16_BE:
-			VOLUME_ADJUST(gint16, GINT16, BE);
-			break;
-		case FMT_U16_BE:
-			VOLUME_ADJUST(guint16, GUINT16, BE);
-			break;
-		case FMT_S8:
-			VOLUME_ADJUST8(gint8);
-			break;
-		case FMT_U8:
-			VOLUME_ADJUST8(guint8);
-			break;
-		default:
-			g_warning("volue_adjust(): unhandled format: %d", fmt);
-			break;
-	}
-}
-
-
 /* transfer data to audio h/w; length is given in bytes
  *
  * data can be modified via effect plugin, rate conversion or
@@ -697,42 +587,9 @@ static void volume_adjust(void* data, int length, AFormat fmt, int channels)
  */
 static void alsa_do_write(gpointer data, int length)
 {
-	EffectPlugin *ep = NULL;
-	int new_freq;
-	int new_chn;
-	AFormat f;
-
 	if (paused)
 		return;
 
-	new_freq = inputf->rate;
-	new_chn = inputf->channels;
-	f = inputf->xmms_format;
-
-	if (effects_enabled() && (ep = get_current_effect_plugin()) &&
-	    ep->query_format)
-		ep->query_format(&f, &new_freq, &new_chn);
-
-	if (f != effectf->xmms_format || (unsigned int)new_freq != effectf->rate ||
-	    (unsigned int)new_chn != effectf->channels)
-	{
-		debug("Changing audio format for effect plugin");
-		g_free(effectf);
-		effectf = snd_format_from_xmms(f, new_freq, new_chn);
-		if (alsa_reopen(effectf) < 0)
-		{
-			/* fatal error... */
-			alsa_close();
-			return;
-		}
-	}
-
-	if (ep)
-		length = ep->mod_samples(&data, length,
-					 inputf->xmms_format,
-					 inputf->rate,
-					 inputf->channels);
-
 	if (alsa_convert_func != NULL)
 		length = alsa_convert_func(convertb, &data, length);
 	if (alsa_stereo_convert_func != NULL)
@@ -742,9 +599,6 @@ static void alsa_do_write(gpointer data, int length)
 						     effectf->rate,
 						     outputf->rate);
 
-	if (alsa_cfg.soft_volume)
-		volume_adjust(data, length, outputf->xmms_format, outputf->channels);
-
 	alsa_write_audio(data, length);
 }
 
@@ -789,11 +643,11 @@ static void alsa_write_audio(char *data, int length)
 		}
 		else
 		{
-			int err = alsa_handle_error((int)written_frames);
+			int err = alsa_recovery((int)written_frames);
 			if (err < 0)
 			{
 				g_warning("alsa_write_audio(): write error: %s",
-					  snd_strerror(-err));
+					  snd_strerror(err));
 				break;
 			}
 		}
@@ -826,7 +680,7 @@ static void *alsa_loop(void *arg)
 	int npfds = snd_pcm_poll_descriptors_count(alsa_pcm);
 	int wr = 0;
 
-	g_mutex_lock(alsa_mutex);
+	g_static_mutex_lock(&alsa_mutex);
 
 	if (npfds <= 0)
 		goto _error;
@@ -845,11 +699,11 @@ static void *alsa_loop(void *arg)
 			}
 			else if (wr < 0)
 			{
-				alsa_handle_error(wr);
+				alsa_recovery(wr);
 			}
 		}
-		else
-			xmms_usleep(10000);
+		else	/* XXX: why is this here? --nenolod */
+			g_usleep(10000);
 
 		if (pause_request != paused)
 			alsa_do_pause(pause_request);
@@ -863,7 +717,7 @@ static void *alsa_loop(void *arg)
 	}
 
  _error:
-	g_mutex_unlock(alsa_mutex);
+	g_static_mutex_unlock(&alsa_mutex);
 	alsa_close_pcm();
 	g_free(thread_buffer);
 	thread_buffer = NULL;
@@ -887,12 +741,12 @@ int alsa_open(AFormat fmt, int rate, int nch)
 		return 0;
 	}
 
-	g_mutex_lock(alsa_mutex);
+	g_static_mutex_lock(&alsa_mutex);
 
 	if (!mixer)
 		alsa_setup_mixer();
 
-	convertb = xmms_convert_buffers_new();
+	convertb = aud_convert_buffers_new();
 
 	output_time_offset = 0;
 	alsa_total_written = alsa_hw_written = 0;
@@ -901,7 +755,7 @@ int alsa_open(AFormat fmt, int rate, int nch)
 	prebuffer = TRUE;
 	remove_prebuffer = FALSE;
 
-	thread_buffer_size = (guint64)cfg.output_buffer_size * inputf->bps / 1000;
+	thread_buffer_size = (guint64)aud_cfg->output_buffer_size * inputf->bps / 1000;
 	if (thread_buffer_size < hw_buffer_size)
 		thread_buffer_size = hw_buffer_size * 2;
 	if (thread_buffer_size < 8192)
@@ -916,7 +770,7 @@ int alsa_open(AFormat fmt, int rate, int nch)
 	pause_request = FALSE;
 	flush_request = -1;
 
-	g_mutex_unlock(alsa_mutex);
+	g_static_mutex_unlock(&alsa_mutex);
 	
 	audio_thread = g_thread_create((GThreadFunc)alsa_loop, NULL, TRUE, NULL);
 	return 1;
@@ -967,10 +821,9 @@ static int format_from_alsa(snd_pcm_format_t fmt)
 static int alsa_setup(struct snd_format *f)
 {
 	int err;
-	snd_pcm_hw_params_t *hwparams;
-	snd_pcm_sw_params_t *swparams;
-	uint alsa_buffer_time;
-	unsigned int alsa_period_time;
+	snd_pcm_hw_params_t *hwparams = NULL;
+	snd_pcm_sw_params_t *swparams = NULL;
+	unsigned int alsa_buffer_time, alsa_period_time;
 	snd_pcm_uframes_t alsa_buffer_size, alsa_period_size;
 
 	debug("alsa_setup");
@@ -989,7 +842,7 @@ static int alsa_setup(struct snd_format *f)
 				SND_PCM_NONBLOCK)) < 0)
 	{
 		g_warning("alsa_setup(): Failed to open pcm device (%s): %s",
-			  alsa_cfg.pcm_device, snd_strerror(-err));
+			  alsa_cfg.pcm_device, snd_strerror(err));
 		alsa_pcm = NULL;
 		g_free(outputf);
 		outputf = NULL;
@@ -1001,12 +854,12 @@ static int alsa_setup(struct snd_format *f)
 
 	if (alsa_cfg.debug)
 	{
-		snd_pcm_info_t *info;
+		snd_pcm_info_t *info = NULL;
 		int alsa_card, alsa_device, alsa_subdevice;
 
 		snd_pcm_info_alloca(&info);
 		snd_pcm_info(alsa_pcm, info);
-		alsa_card =  snd_pcm_info_get_card(info);
+		alsa_card = snd_pcm_info_get_card(info);
 		alsa_device = snd_pcm_info_get_device(info);
 		alsa_subdevice = snd_pcm_info_get_subdevice(info);
 		printf("Card %i, Device %i, Subdevice %i\n",
@@ -1018,7 +871,7 @@ static int alsa_setup(struct snd_format *f)
 	if ((err = snd_pcm_hw_params_any(alsa_pcm, hwparams)) < 0)
 	{
 		g_warning("alsa_setup(): No configuration available for "
-			  "playback: %s", snd_strerror(-err));
+			  "playback: %s", snd_strerror(err));
 		return -1;
 	}
 
@@ -1026,7 +879,7 @@ static int alsa_setup(struct snd_format *f)
 						SND_PCM_ACCESS_RW_INTERLEAVED)) < 0)
 	{
 		g_warning("alsa_setup(): Cannot set direct write mode: %s",
-			  snd_strerror(-err));
+			  snd_strerror(err));
 		return -1;
 	}
 
@@ -1057,7 +910,7 @@ static int alsa_setup(struct snd_format *f)
 			debug("Converting format from %d to %d",
 			      f->xmms_format, outputf->xmms_format);
 			alsa_convert_func =
-				xmms_convert_get_func(outputf->xmms_format,
+				aud_convert_get_func(outputf->xmms_format,
 						      f->xmms_format);
 			if (alsa_convert_func == NULL)
 				return -1;
@@ -1066,7 +919,7 @@ static int alsa_setup(struct snd_format *f)
 		{
 			g_warning("alsa_setup(): Sample format not "
 				  "available for playback: %s",
-				  snd_strerror(-err));
+				  snd_strerror(err));
 			return -1;
 		}
 	}
@@ -1077,7 +930,7 @@ static int alsa_setup(struct snd_format *f)
 		debug("Converting channels from %d to %d",
 		      f->channels, outputf->channels);
 		alsa_stereo_convert_func =
-			xmms_convert_get_channel_func(outputf->xmms_format,
+			aud_convert_get_channel_func(outputf->xmms_format,
 						      outputf->channels,
 						      f->channels);
 		if (alsa_stereo_convert_func == NULL)
@@ -1095,7 +948,7 @@ static int alsa_setup(struct snd_format *f)
 		debug("Converting samplerate from %d to %d",
 		      f->rate, outputf->rate);
 		alsa_frequency_convert_func =
-			xmms_convert_get_frequency_func(outputf->xmms_format,
+			aud_convert_get_frequency_func(outputf->xmms_format,
 							outputf->channels);
 		if (alsa_frequency_convert_func == NULL)
 			return -1;
@@ -1109,7 +962,7 @@ static int alsa_setup(struct snd_format *f)
 							  &alsa_buffer_time, 0)) < 0)
 	{
 		g_warning("alsa_setup(): Set buffer time failed: %s.",
-			  snd_strerror(-err));
+			  snd_strerror(err));
 		return -1;
 	}
 
@@ -1118,7 +971,7 @@ static int alsa_setup(struct snd_format *f)
 							  &alsa_period_time, 0)) < 0)
 	{
 		g_warning("alsa_setup(): Set period time failed: %s.",
-			  snd_strerror(-err));
+			  snd_strerror(err));
 		return -1;
 	}
 
@@ -1134,7 +987,7 @@ static int alsa_setup(struct snd_format *f)
 	{
 		g_warning("alsa_setup(): snd_pcm_hw_params_get_buffer_size() "
 			  "failed: %s",
-			  snd_strerror(-err));
+			  snd_strerror(err));
 		return -1;
 	}
 
@@ -1142,20 +995,20 @@ static int alsa_setup(struct snd_format *f)
 	{
 		g_warning("alsa_setup(): snd_pcm_hw_params_get_period_size() "
 			  "failed: %s",
-			  snd_strerror(-err));
+			  snd_strerror(err));
 		return -1;
 	}
 
 	alsa_can_pause = snd_pcm_hw_params_can_pause(hwparams);
+	debug("can pause: %d", alsa_can_pause);
 
 	snd_pcm_sw_params_alloca(&swparams);
 	snd_pcm_sw_params_current(alsa_pcm, swparams);
 
-	/* This has effect for non-mmap only */
 	if ((err = snd_pcm_sw_params_set_start_threshold(alsa_pcm,
 			swparams, alsa_buffer_size - alsa_period_size) < 0))
 		g_warning("alsa_setup(): setting start "
-			  "threshold failed: %s", snd_strerror(-err));
+			  "threshold failed: %s", snd_strerror(err));
 	if (snd_pcm_sw_params(alsa_pcm, swparams) < 0)
 	{
 		g_warning("alsa_setup(): Unable to install sw params");
