@@ -6,16 +6,16 @@
  *	This code maps xmms calls into the jack translation library
  */
 
-#include "audacious/configdb.h"
-#include "audacious/util.h"
+#include <audacious/plugin.h>
+#include <audacious/configdb.h>
+#include <audacious/util.h>
 #include <dlfcn.h>
 #include <gtk/gtk.h>
-#include <glib/gi18n.h>
+#include <audacious/i18n.h>
 #include <stdio.h>
 #include "config.h"
 #include "bio2jack.h" /* includes for the bio2jack library */
 #include "jack.h"
-#include "xconvert.h" /* xmms rate conversion header file */
 #include <string.h>
 
 
@@ -58,12 +58,6 @@ static format_info_t output;
 static convert_freq_func_t freq_convert; /* rate convert function */
 static struct xmms_convert_buffers *convertb; /* convert buffer */
 
-#define MAKE_FUNCPTR(f) static typeof(f) * fp_##f = NULL;
-MAKE_FUNCPTR(xmms_convert_buffers_new);
-MAKE_FUNCPTR(xmms_convert_buffers_destroy);
-MAKE_FUNCPTR(xmms_convert_get_frequency_func);
-void *xmmslibhandle; /* handle to the dlopen'ed libxmms.so */
-
 static int isXmmsFrequencyAvailable = 0;
 
 static gboolean output_opened; /* true if we have a connection to jack */
@@ -88,8 +82,7 @@ void jack_cleanup(void)
   /* only clean this up if we have the function to call */
   if(isXmmsFrequencyAvailable)
   {
-    fp_xmms_convert_buffers_destroy(convertb); /* clean up the rate conversion buffers */
-    dlclose(xmmslibhandle);
+    aud_convert_buffers_destroy(convertb); /* clean up the rate conversion buffers */
   }
 
   return;
@@ -99,9 +92,9 @@ void jack_cleanup(void)
 void jack_sample_rate_error(void)
 {
 	dialog = gtk_dialog_new();
-	gtk_window_set_title(GTK_WINDOW(dialog), ("Sample rate mismatch"));
+	gtk_window_set_title(GTK_WINDOW(dialog), _("Sample rate mismatch"));
 	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
-	label = gtk_label_new((
+	label = gtk_label_new(_(
 		"Xmms is asking for a sample rate that differs from\n "
 		"that of the jack server.  Xmms 1.2.8 or later\n"
 		"contains resampling routines that xmms-jack will\n"
@@ -115,7 +108,7 @@ void jack_sample_rate_error(void)
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
 	gtk_widget_show(label);
 
-	button = gtk_button_new_with_label((" Close "));
+	button = gtk_button_new_with_label(_(" Close "));
 	gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(dialog));
 	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
@@ -142,7 +135,7 @@ gint jack_get_written_time(void)
 gint jack_get_output_time(void)
 {
   gint return_val;
-  
+
   /* don't try to get any values if the device is still closed */
   if(JACK_GetState(driver) == CLOSED)
     return_val = 0;
@@ -182,7 +175,7 @@ void jack_set_port_connection_mode()
 {
   /* setup the port connection mode that determines how bio2jack will connect ports */
   enum JACK_PORT_CONNECTION_MODE mode;
-  
+
   if(strcmp(jack_cfg.port_connection_mode, "CONNECT_ALL") == 0)
       mode = CONNECT_ALL;
   else if(strcmp(jack_cfg.port_connection_mode, "CONNECT_OUTPUT") == 0)
@@ -203,7 +196,7 @@ void jack_init(void)
   /* read the isTraceEnabled setting from the config file */
   ConfigDb *cfgfile;
 
-  cfgfile = bmp_cfg_db_open();
+  cfgfile = aud_cfg_db_open();
   if (!cfgfile)
   {
       jack_cfg.isTraceEnabled = FALSE;
@@ -212,16 +205,16 @@ void jack_init(void)
       jack_cfg.volume_right = 25;
   } else
   {
-      bmp_cfg_db_get_bool(cfgfile, "jack", "isTraceEnabled", &jack_cfg.isTraceEnabled);
-      if(!bmp_cfg_db_get_string(cfgfile, "jack", "port_connection_mode", &jack_cfg.port_connection_mode))
+      aud_cfg_db_get_bool(cfgfile, "jack", "isTraceEnabled", &jack_cfg.isTraceEnabled);
+      if(!aud_cfg_db_get_string(cfgfile, "jack", "port_connection_mode", &jack_cfg.port_connection_mode))
           jack_cfg.port_connection_mode = "CONNECT_ALL";
-      if(!bmp_cfg_db_get_int(cfgfile, "jack", "volume_left", &jack_cfg.volume_left))
+      if(!aud_cfg_db_get_int(cfgfile, "jack", "volume_left", &jack_cfg.volume_left))
           jack_cfg.volume_left = 25;
-      if(!bmp_cfg_db_get_int(cfgfile, "jack", "volume_right", &jack_cfg.volume_right))
+      if(!aud_cfg_db_get_int(cfgfile, "jack", "volume_right", &jack_cfg.volume_right))
           jack_cfg.volume_right = 25;
   }
 
-  bmp_cfg_db_close(cfgfile);
+  aud_cfg_db_close(cfgfile);
 
 
   TRACE("initializing\n");
@@ -234,52 +227,13 @@ void jack_init(void)
   /* set the port connection mode */
   jack_set_port_connection_mode();
 
-  /* XXX unportable to 2.x */
-  xmmslibhandle = dlopen("libaudacious.so", RTLD_NOW);
-  if(xmmslibhandle)
-  {
-    fp_xmms_convert_buffers_new = dlsym(xmmslibhandle, "xmms_convert_buffers_new");
-    fp_xmms_convert_buffers_destroy = dlsym(xmmslibhandle, "xmms_convert_buffers_destroy");
-    fp_xmms_convert_get_frequency_func = dlsym(xmmslibhandle, "xmms_convert_get_frequency_func");
-
-    if(!fp_xmms_convert_buffers_new)
-    {
-      TRACE("fp_xmms_convert_buffers_new couldn't be dlsym'ed\n");
-      TRACE("dlerror: %s\n", dlerror());
-    }
-
-    if(!fp_xmms_convert_buffers_destroy)
-    {
-      TRACE("fp_xmms_convert_buffers_destroy couldn't be dlsym'ed\n");
-      TRACE("dlerror: %s\n", dlerror());
-    }
-
-    if(!fp_xmms_convert_get_frequency_func)
-    {
-      TRACE("fp_xmms_get_frequency_func couldn't be dlsym'ed\n");
-      TRACE("dlerror: %s\n", dlerror());
-    }
-
-    if(!fp_xmms_convert_buffers_new || !fp_xmms_convert_buffers_destroy ||
-       !fp_xmms_convert_get_frequency_func)
-    {
-      dlclose(xmmslibhandle); /* close the library, no need to keep it open */
-      TRACE("One or more frequency convertion functions are missing, upgrade to xmms >=1.2.8\n");
-    } else
-    {
-      TRACE("Found frequency convertion functions, setting isXmmsFrequencyAvailable to 1\n");
-      isXmmsFrequencyAvailable = 1;
-    }
-  } else
-  {
-    TRACE("unable to dlopen '%s'\n", "libaudacious.so");
-  }
+  isXmmsFrequencyAvailable = 1;
 
   /* only initialize this stuff if we have the functions available */
   if(isXmmsFrequencyAvailable)
   {
-    convertb = fp_xmms_convert_buffers_new ();
-    freq_convert = fp_xmms_convert_get_frequency_func(FMT_S16_LE, 2);
+    convertb = aud_convert_buffers_new ();
+    freq_convert = aud_convert_get_frequency_func(FMT_S16_LE, 2);
   }
 
   output_opened = FALSE;
@@ -320,10 +274,10 @@ void jack_close(void)
 {
   ConfigDb *cfgfile;
 
-  cfgfile = bmp_cfg_db_open();
-  bmp_cfg_db_set_int(cfgfile, "jack", "volume_left", jack_cfg.volume_left); /* stores the volume setting */
-  bmp_cfg_db_set_int(cfgfile, "jack", "volume_right", jack_cfg.volume_right);
-  bmp_cfg_db_close(cfgfile);
+  cfgfile = aud_cfg_db_open();
+  aud_cfg_db_set_int(cfgfile, "jack", "volume_left", jack_cfg.volume_left); /* stores the volume setting */
+  aud_cfg_db_set_int(cfgfile, "jack", "volume_right", jack_cfg.volume_right);
+  aud_cfg_db_close(cfgfile);
 
   TRACE("\n");
 
@@ -429,68 +383,9 @@ gint jack_open(AFormat fmt, gint sample_rate, gint num_channels)
 void jack_write(gpointer ptr, gint length)
 {
   long written;
-  EffectPlugin *ep;
-  AFormat new_format;
-  int new_frequency, new_channels;
-  long positionMS;
 
   TRACE("starting length of %d\n", length);
 
-  /* copy the current values into temporary values */
-  new_format = input.format;
-  new_frequency = input.frequency;
-  new_channels = input.channels;
-
-  /* query xmms for the current plugin */
-  ep = get_current_effect_plugin();
-  if(effects_enabled() && ep && ep->query_format)
-  {
-    ep->query_format(&new_format, &new_frequency, &new_channels);
-  }
-
-  /* if the format has changed take this into account by modifying */
-  /* the time offset and reopening the device with the new format settings */
-  if (new_format != effect.format ||
-      new_frequency != effect.frequency ||
-      new_channels != effect.channels)
-  {
-    TRACE("format changed, storing new values and opening/closing jack\n");
-    TRACE("effect.format == %d, new_format == %d, effect.frequency == %ld, new_frequency == %d, effect.channels == %d, new_channels = %d\n",
-	  effect.format, new_format, effect.frequency, new_frequency, effect.channels, new_channels);
- 
-    positionMS = JACK_GetPosition(driver, MILLISECONDS, PLAYED);
-
-    jack_close();
-    jack_open(new_format, new_frequency, new_channels);
- 
-    /* restore the position after the open and close */
-    JACK_SetState(driver, PAUSED);
-    JACK_SetPosition(driver, MILLISECONDS, positionMS);
-    JACK_SetState(driver, PLAYING);
-  }
-
-  /* if effects are enabled and we have a plugin, run the current */
-  /* samples through the plugin */
-  if (effects_enabled() && ep && ep->mod_samples)
-  {
-    length = ep->mod_samples(&ptr, length,
-                             input.format,
-                             input.frequency,
-                             input.channels);
-    TRACE("effects_enabled(), length is now %d\n", length);
-  }
-
-  TRACE("effect.frequency == %ld, input.frequency == %ld, output.frequency == %ld\n",
-        effect.frequency, input.frequency, output.frequency);
-
-  /* if we need rate conversion, perform it here */
-  if((effect.frequency != output.frequency) && isXmmsFrequencyAvailable)
-  {
-    TRACE("performing rate conversion from '%ld'(effect) to '%ld'(output)\n", effect.frequency, output.frequency);
-    length = freq_convert (convertb, &ptr, length, effect.frequency, output.frequency);
-  }
-
-  TRACE("length = %d\n", length);
   /* loop until we have written all the data out to the jack device */
   /* this is due to xmms' audio driver api */
   char *buf = (char*)ptr;
@@ -587,10 +482,10 @@ void jack_get_volume(int *l, int *r)
 void jack_about(void)
 {
 	static GtkWidget *aboutbox = NULL;
-	
+
 	if ( aboutbox == NULL )
 	{
-		aboutbox = xmms_show_message(
+		aboutbox = audacious_info_dialog(
 			_("About JACK Output Plugin 0.17"),
 			_("XMMS jack Driver 0.17\n\n"
 			  "xmms-jack.sf.net\nChris Morgan<cmorgan@alum.wpi.edu>\n\n"
@@ -611,29 +506,25 @@ jack_tell_audio(AFormat * fmt, gint * srate, gint * nch)
 
 OutputPlugin jack_op =
 {
-	NULL,
-	NULL,
-	"JACK Output Plugin 0.17",
-	jack_init,
-	jack_cleanup,
-	jack_about,
-	jack_configure,
-	jack_get_volume,
-	jack_set_volume,
-	jack_open,
-	jack_write,
-	jack_close,
-	jack_flush,
-	jack_pause,
-	jack_free,
-	jack_playing,
-	jack_get_output_time,
-	jack_get_written_time,
-	jack_tell_audio
+	.description = "JACK Output Plugin 0.17",
+	.init = jack_init,
+	.cleanup = jack_cleanup,
+	.about = jack_about,
+	.configure = jack_configure,
+	.get_volume = jack_get_volume,
+	.set_volume = jack_set_volume,
+	.open_audio = jack_open,
+	.write_audio = jack_write,
+	.close_audio = jack_close,
+	.flush = jack_flush,
+	.pause = jack_pause,
+	.buffer_free = jack_free,
+	.buffer_playing = jack_playing,
+	.output_time = jack_get_output_time,
+	.written_time = jack_get_written_time,
+	.tell_audio = jack_tell_audio
 };
 
+OutputPlugin *jack_oplist[] = { &jack_op, NULL };
 
-OutputPlugin *get_oplugin_info(void)
-{
-	return &jack_op;
-}
+SIMPLE_OUTPUT_PLUGIN(jack, jack_oplist);
