@@ -23,15 +23,17 @@
 #define WAV_SNDFILE_H
 
 #define BUFFER_SIZE 			8192
-#define	TITLE_LEN			64
 
 static	void 	plugin_init (void);
+static	void 	plugin_cleanup (void);
 static	int	is_our_file (char *filename);
-static	void 	play_start (char *filename);
-static	void 	play_stop (void);
-static	void 	file_seek (int time);
-static	int	get_time (void);
-static	void 	get_song_info (char *filename, char **title, int *length);
+static	void 	play_start (InputPlayback *playback);
+static	void 	play_stop (InputPlayback *playback);
+static	void 	file_seek (InputPlayback *playback, gint time);
+static	void 	get_song_info (gchar *filename, gchar **title, gint *length);
+static	Tuple*  get_song_tuple (gchar *filename);
 static  void    wav_about (void);
+static	void	play_pause (InputPlayback *playback, gshort p);
+static  void file_mseek (InputPlayback *playback, gulong millisecond);
 
 #endif
